#if defined _scripter_functions_included
	#endinput
#endif
#define _scripter_functions_included

/////

#include <sdktools>

//	#define функции/константы.
#include <scripter_functions/sf_define>

//	Оружие.
#include <scripter_functions/sf_item_info>

//	Игрок.
#include <scripter_functions/sf_client>

/////

/**
 *	true = таймер уничтожен (был запущен).
 */
stock bool SF_KillTimer(Handle &t)
{
	if (t)
	{
		KillTimer(t);
		t = null;
		return true;
	}
	return false;
}
	
/**
 *	true = Handle уничтожено (было валидным).
 *	@note				"delete" не используется, т.к. меняет значение на null, противореча C++.
 */
stock bool SF_CloseHandle(Handle &h)
{
	if (h)
	{
		CloseHandle(h);
		h = null;
		return true;
	}
	return false;
}

/**
 *	Если в flags нет flag, то flag добавляется и возвращает true.
 *	@note				Для flag можно указать несколько флагов.
 */
stock bool SF_AddFlag(int &flags, int flag)
{
	if (!HAS_FLAGS(flags, flag))
	{
		flags |= flag;
		return true;
	}
	return false;
}

/**
 *	Если в flags есть flag, то flag удаляется и возвращает true.
 *	@note				Для flag можно указать несколько флагов.
 */
stock bool SF_RemoveFlag(int &flags, int flag)
{
	if (HAS_FLAGS(flags, flag))
	{
		flags &= ~flag;
		return true;
	}
	return false;
}

/**
 *	Добавить звук в загрузку и кэшировать.
 *
 *	@note				"sound/quake/1.mp3" или "quake/1.mp3" - поддерживаются оба варианта.
 *						Все \ заменяются на /.
 *
 *	@param convar		Переменная, хранящая путь к звуку. Указывать необязательно, достаточно @param sound. Вызывайте из OnConfigsExecuted(), а не OnMapStart().
 *	@param sound		Путь к звуку. Указывать необязательно, достаточно @param convar.
 *	@param buffer		Записать в переменную успешно добавленный в загрузку и кэшированный звук (записывается без "sound/").
 *	@param buffer_size	Размер @param buffer.
 *
 *	@return				true = звук добавлен в загрузку и кэширован.
 */
stock bool SF_AddSoundToDownloadAndCache(ConVar convar=null, const char[] sound="", char[] buffer="", int buffer_size=0)
{
	if (convar || sound[0])
	{
		char s[128];
		s[0] = 0;
		
		if (sound[0]) {
			strcopy(SZF(s), sound);
		}
		else{
			convar.GetString(SZF(s));
		}
		
		int i;
		
		if (TrimString(s) > 4/* "x.mp3" */ && (i = FindCharInString(s, '.', /* reverse */true)) > 0 && (!strcmp(s[i+1], "mp3") || !strcmp(s[i+1], "wav")))
		{
			ReplaceString(SZF(s), "\\", "/");

			if (strncmp(s, "sound/", /* num */6, /* caseSensitive */false) != 0) {
				Format(SZF(s), "sound/%s", s);
			}
			
			AddFileToDownloadsTable(s);
			static EngineVersion s_EngineVersion = Engine_Unknown;
			
			if (s_EngineVersion == Engine_Unknown) {
				s_EngineVersion = GetEngineVersion();
			}
			
			if (s_EngineVersion == Engine_CSGO)
			{
				Format(SZF(s), "*%s", s[6]);
				static int s_soundprecache = INVALID_STRING_TABLE;
				
				if (s_soundprecache != INVALID_STRING_TABLE || (s_soundprecache = FindStringTable("soundprecache")) != INVALID_STRING_TABLE) {
					AddToStringTable(s_soundprecache, s);
				}
				else{
					SetFailState("INVALID_STRING_TABLE 'soundprecache'");
				}
				
				if (buffer_size) {
					return strcopy(buffer, buffer_size, s	) > 0;
				}
			}
			else
			{
				PrecacheSound(s[6]);
				
				if (buffer_size) {
					return strcopy(buffer, buffer_size, s[6]) > 0;
				}
			}
		}
		else if (sound[0] && convar) {
			SF_AddSoundToDownloadAndCache(convar); // Звук в @param sound оказался неверным, попробуем вытащить его из @param convar.
		}
	}
	
	if (buffer_size) {
		buffer[0] = 0;
	}
	
	return false;
}

/**
 *	Анализировать файл.
 *
 *	@note				Используем, когда важен регистр ключей, или надоело рассказывать про utf8 без bom, ибо KeyValues пьян.
 *
 *	@param ns			Вызывается, когда парсер входит в новый раздел или подраздел.
 *	@param kv			Вызывается, когда синтаксический анализатор находит новую пару ключ/значение.
 *	@param ns			Вызывается, когда парсер находит конец текущего раздела.
 *
 *	@return				true = файл успешно проанализирован.
 */
stock bool SF_ParseFile(const char[] file, SMC_NewSection ns=INVALID_FUNCTION, SMC_KeyValue kv=INVALID_FUNCTION, SMC_EndSection es=INVALID_FUNCTION)
{
	if (file[0] && ((kv != INVALID_FUNCTION) || (ns != INVALID_FUNCTION) || (es != INVALID_FUNCTION)))
	{
		SMCParser smc  = new SMCParser();
		
		SMC_SetReaders(smc, ns, kv, es );
		SMCError error = smc.ParseFile(file);
		
		if (error == SMCError_Okay)
		{
			delete smc ;
			return true;
		}
		
		char error_text[256];
		smc.GetErrorString(error, SZF(error_text));
		
		delete smc;
		
		LogError("%s", error_text);
	}
	return false;
}

stock bool SF_StringToColor(const char[] str, int[] color, const int color_size)
{
	if (str[0] && ((color_size == 3) || (color_size == 4)))
	{
		char s[128];
		int i = strcopy(SZF(s), str);
		
		if (i >= (color_size == 4 ? 7 : 5))
		{
			bool  number_found	= false; // "xxY xxY xxY xxY" Y было найдено.
			
			int[] clr			= new int[color_size];
			int   clr_size		= color_size;
			
			while (i--)
			{
				if (number_found)
				{
					if (IsCharSpace(s[i]))
					{
						clr[--clr_size] = StringToInt(s[i]);
						number_found = false;
						s[i] = 0;
						
						if (clr_size == 0)
						{
							if (TrimString(s)) {
								return false; // Все числа получили, но там дальше есть ещё что-то, кроме пробелов.
							}
							
							break;
						}
					}
					else if (IsCharNumeric(s[i]))
					{
						if (i == 0)
						{
							clr[--clr_size] = StringToInt(s[i]);
							break;
						}
					}
					else{
						return false;
					}
				}
				else if (IsCharNumeric(s[i]))
				{
					if (i == 0)
					{
						clr[--clr_size] = StringToInt(s[i]);
						break;
					}
					number_found = true;
				}
				else if (!IsCharSpace(s[i])) {
					return false;
				}
			}

			if (	(clr_size == 0)
				&&	(clr[0] >= 0) && (clr[0] <= 255)
				&&	(clr[1] >= 0) && (clr[1] <= 255)
				&&	(clr[2] >= 0) && (clr[2] <= 255)
				&&	((color_size != 4) || ((clr[3] >= 0) && (clr[3] <= 255)))	)
			{
				color[0] = clr[0];
				color[1] = clr[1];
				color[2] = clr[2];
				
				if (color_size == 4) {
					color[3] = clr[3];
				}
				
				return true;
			}
		}
	}
	return false;
}

stock bool SF_ConVarToColor(ConVar convar, int[] color, const int color_size)
{
	char s[128];
	convar.GetString(SZF(s));
	return SF_StringToColor(s, color, color_size);
}

stock int SF_GetCurrentMap(char[] buffer, int maxlength)
{
	int bytes = GetCurrentMap(buffer, maxlength);
	if (bytes > 0) {
		int i = bytes;
		while (--i > -1) {
			if ((buffer[i] == '/') || (buffer[i] == '\\')) {
				return strcopy(buffer, maxlength, buffer[i + 1]);
			}
		}
	}
	return bytes;
}

/**
 *	Стандартная функция ReadFlagString, встретив символ, не являющийся флагом, останавливается.
 *
 *	@param sFlags		Админ флаги "abcdef" и т.д.
 *	@param flagCount	Кол-во валидных неповторяющихся флагов в @param sFlags строке.
 *
 *	@return				ADMFLAG_ биты или 0.
 */
stock int SF_ReadFlagString(const char[] sFlags, int &flagCount=0)
{
	AdminFlag adminFlag;
	
	int bits  = 0;
	flagCount = 0;
	
	for (int i = 0, length = strlen(sFlags), bit; i < length; i++) {
		if (FindFlagByChar(sFlags[i], adminFlag) && (bit = FlagToBit(adminFlag)) && SF_AddFlag(bits, bit)) {
			++flagCount;
		}
	}
	
	return bits;
}