enum struct ItemInfo
{
	int id;					// CSWeaponID (cstrike.inc). Не "CSWeaponID id", чтобы "cstrike" зря не требовалось. Используйте view_as<CSWeaponID>(id).
	
	int ammoType;			// Тип патронов (m_iPrimaryAmmoType). -1 = не поддерживается.
	int ammoClip;			// Патронов в обоиме (m_iClip1).      -1 = не поддерживается.
	int ammoReserve;		// Патронов в запасе (m_iPrimaryReserveAmmoCount csgo. m_iAmmo + m_iPrimaryAmmoType css.). От 0 и выше.
	
	int slot;				// Слот (GetPlayerWeaponSlot).
	int price;				// Стоимость в меню покупки (0 = бесплатное).
	
	int definitionIndex;	// m_iItemDefinitionIndex (csgo). В css всегда 0.
}

/**
 *	Получить информацию об оружии.
 *
 *	@note					classname должно начинаться с "weapon_".
 *	@param sItem			Название оружия БЕЗ "weapon_".
 *	@param item				ItemInfo объект, в который будет записана информация.
 *	@return					true = информация получена. false = оружия нет в списке, т.к. не поддерживается или не добавлено.
 */
stock bool SF_GetItemInfo(const char[] sItem, ItemInfo item)
{
	// Прячем переменную тут, чтобы не занимала память, если plugin не использует SF_GetItemInfo.
	// Хотя,  при компиляции будет варнинг о неиспользуемой переменной и её вырежет, но зачем нам варнинги.
	static StringMap s_Trie = null;
	
	if(!s_Trie)
	{
		s_Trie = new StringMap();
		Init(s_Trie);
	}
	
	return s_Trie.GetArray(sItem, SZF(item));
}

static void Add(StringMap trie, const char[] sItem, const int data[7])
{
	ItemInfo item;
	
	item.id					= data[0];
	item.ammoType			= data[1];
	item.ammoClip			= data[2];
	item.ammoReserve		= data[3];
	item.slot				= data[4];
	item.price				= data[5];
	item.definitionIndex	= data[6];
	
	trie.SetArray(sItem, SZF(item));
}

static void Init(StringMap trie)
{
	switch (GetEngineVersion())
	{
		case Engine_CSGO:
		{
			Add(trie, "m4a1", 					{21,3,30,90,0,3100,16});
			Add(trie, "m4a1_silencer",			{60,4,25,75,0,2900,60});
			
			Add(trie, "mp7", 					{42,7,30,120,0,1500,33});
			Add(trie, "mp5sd", 					{18,7,30,120,0,1500,23});
			
			Add(trie, "ak47", 					{27,2,30,90,0,2700,7});
			Add(trie, "aug", 					{8,2,30,90,0,3300,8});
			Add(trie, "awp", 					{17,6,10,30,0,4750,9});
			Add(trie, "bizon", 					{35,7,64,120,0,1400,26});
			Add(trie, "famas", 					{15,3,25,90,0,2050,10});
			Add(trie, "g3sg1", 					{23,2,20,90,0,5000,11});
			Add(trie, "galilar", 				{34,3,35,90,0,1800,13});
			Add(trie, "m249", 					{19,5,100,200,0,5200,14});
			Add(trie, "mac10", 					{7,9,30,100,0,1050,17});
			Add(trie, "mag7", 					{36,8,5,32,0,1300,27});
			Add(trie, "mp9", 					{43,7,30,120,0,1250,34});
			Add(trie, "negev", 					{37,5,150,300,0,1700,28});
			Add(trie, "nova", 					{44,8,8,32,0,1050,35});
			Add(trie, "p90", 					{29,13,50,100,0,2350,19});
			Add(trie, "sawedoff", 				{38,8,7,32,0,1100,29});
			Add(trie, "scar20", 				{47,2,20,90,0,5000,38});
			Add(trie, "sg556", 					{48,3,30,90,0,3000,39});
			Add(trie, "ssg08", 					{49,2,10,90,0,1700,40});
			Add(trie, "ump45", 					{12,9,25,100,0,1200,24});
			Add(trie, "xm1014", 				{5,8,7,32,0,2000,25});
			
			/////
			
			Add(trie, "cz75a", 					{63,12,12,12,1,500,63});
			Add(trie, "fiveseven", 				{11,13,20,100,1,500,3});
			
			// Если в инвентаре экипирован usp_silencer, то GivePlayerItem выдаст hkp2000.
			// Если в инвентаре экипирован hkp2000     , то выдаётся и hkp2000 и  usp_silencer.
			Add(trie, "usp_silencer", 			{61,11,12,24,1,200,61});	// GetEntityClassname = "weapon_hkp2000".
			Add(trie, "hkp2000", 				{41,10,13,52,1,200,32});
			
			Add(trie, "deagle", 				{25,1,7,35,1,700,1});
			Add(trie, "revolver", 				{64,1,8,8,1,600,64});
			
			Add(trie, "elite", 					{10,7,30,120,1,300,2});
			Add(trie, "glock", 					{2,7,20,120,1,200,4});
			Add(trie, "p250", 					{45,20,13,26,1,300,36});
			Add(trie, "tec9", 					{39,7,18,90,1,500,30});
			
			/////
			
			Add(trie, "decoy", 					{52,18,-1,0,3,50,47});
			Add(trie, "diversion", 				{82,18,-1,0,3,50,82});		// GetEntityClassname = "weapon_decoy".
			
			Add(trie, "molotov", 				{51,17,-1,0,3,400,46});
			Add(trie, "firebomb", 				{81,17,-1,0,3,400,81});		// GetEntityClassname = "weapon_molotov".
			
			Add(trie, "hegrenade", 				{4,14,-1,0,3,300,44});
			Add(trie, "frag_grenade",			{83,14,-1,0,3,300,83});		// GetEntityClassname = "weapon_hegrenade".
			
			Add(trie, "flashbang", 				{24,15,-1,0,3,200,43});
			Add(trie, "smokegrenade", 			{9,16,-1,0,3,300,45});
			Add(trie, "incgrenade", 			{53,17,-1,0,3,600,48});
			Add(trie, "tagrenade", 				{68,22,-1,0,3,100,68});
			Add(trie, "snowball", 				{84,25,-1,0,3,100,84});
			
			/////
			
			Add(trie, "c4", 					{6,-1,-1,0,4,0,49});
			Add(trie, "bumpmine", 				{85,24,3,0,4,300,85});
			
			/////
			
			Add(trie, "tablet", 				{72,-1,-1,0,12,300,72});
			Add(trie, "healthshot", 			{57,21,-1,0,11,0,57});
			
			// Падают на землю (наверно зависит от карты) при GivePlayerItem, но работает CreateEntityByName + EquipPlayerWeapon:
			Add(trie, "shield", 				{30,-1,-1,0,11,1100,37});
			Add(trie, "zone_repulsor", 			{0,-1,-1,0,11,0,0});
			
			/////
			
			Add(trie, "taser", 					{40,19,1,0,2,200,31});
			
			Add(trie, "knife", 					{28,-1,-1,0,2,0,42});
			Add(trie, "knifegg", 				{50,-1,-1,0,2,0,41});
			Add(trie, "knife_t", 				{59,-1,-1,0,2,0,59});		// GetEntityClassname = "weapon_knife".
			
			// Падают на землю (наверно зависит от карты) при GivePlayerItem, но работает CreateEntityByName + EquipPlayerWeapon:
			
			Add(trie, "axe", 					{75,-1,-1,0,2,0,75});		// GetEntityClassname = "weapon_melee".
			Add(trie, "melee", 					{74,-1,-1,0,2,0,74});
			
			Add(trie, "fists", 					{69,-1,-1,0,2,0,69});		// Работает только CreateEntityByName + EquipPlayerWeapon (наверно зависит от карты).
			Add(trie, "hammer", 				{76,-1,-1,0,2,0,76});
			Add(trie, "spanner", 				{78,-1,-1,0,2,0,78});
			
			// Если в core.cfg "FollowCSGOServerGuidelines" "yes", то при GivePlayerItem все ножи ниже (кроме: bayonet) выдают ошибку.
			// Если выдать через sv_cheats 1 + give или CreateEntityByName + EquipPlayerWeapon, то ошибки нет.
			
			Add(trie, "knife_ghost", 			{80,-1,-1,0,2,0,80});		// GetEntityClassname = "weapon_knife".
			
			// Падают на землю при GivePlayerItem, а CreateEntityByName не работает:
			
			Add(trie, "bayonet", 				{500,-1,-1,0,2,0,500});
			
			Add(trie, "knife_css", 				{503,-1,-1,0,2,0,503});
			Add(trie, "knife_flip", 			{505,-1,-1,0,2,0,505});
			Add(trie, "knife_gut", 				{506,-1,-1,0,2,0,506});
			Add(trie, "knife_karambit", 		{507,-1,-1,0,2,0,507});
			Add(trie, "knife_m9_bayonet", 		{508,-1,-1,0,2,0,508});
			Add(trie, "knife_tactical", 		{509,-1,-1,0,2,0,509});
			Add(trie, "knife_falchion", 		{512,-1,-1,0,2,0,512});
			Add(trie, "knife_survival_bowie", 	{514,-1,-1,0,2,0,514});
			Add(trie, "knife_butterfly", 		{515,-1,-1,0,2,0,515});
			Add(trie, "knife_push", 			{516,-1,-1,0,2,0,516});
			Add(trie, "knife_cord", 			{517,-1,-1,0,2,0,517});
			Add(trie, "knife_canis", 			{518,-1,-1,0,2,0,518});
			Add(trie, "knife_ursus", 			{519,-1,-1,0,2,0,519});
			Add(trie, "knife_gypsy_jackknife",	{520,-1,-1,0,2,0,520});
			Add(trie, "knife_outdoor", 			{521,-1,-1,0,2,0,521});
			Add(trie, "knife_stiletto", 		{522,-1,-1,0,2,0,522});
			Add(trie, "knife_widowmaker", 		{523,-1,-1,0,2,0,523});
			Add(trie, "knife_skeleton", 		{525,-1,-1,0,2,0,525});
		}
		
		case Engine_CSS, Engine_SourceSDK2006:
		{
			Add(trie, "ak47", 					{27,2,30,90,0,2500,0});
			Add(trie, "aug", 					{8,2,30,90,0,3500,0});
			Add(trie, "awp", 					{17,5,10,30,0,4750,0});
			Add(trie, "deagle", 				{25,1,7,35,1,650,0});
			Add(trie, "elite", 					{10,6,30,120,1,800,0});
			Add(trie, "famas", 					{15,3,25,90,0,2250,0});
			Add(trie, "fiveseven", 				{11,10,20,100,1,750,0});
			Add(trie, "g3sg1", 					{23,2,20,90,0,5000,0});
			Add(trie, "galil", 					{14,3,35,90,0,2000,0});
			Add(trie, "glock", 					{2,6,20,120,1,400,0});
			Add(trie, "m249", 					{19,4,100,200,0,5750,0});
			Add(trie, "m3", 					{20,7,8,32,0,1700,0});
			Add(trie, "m4a1", 					{21,3,30,90,0,3100,0});
			Add(trie, "mac10", 					{7,8,30,100,0,1400,0});
			Add(trie, "mp5navy", 				{18,6,30,120,0,1500,0});
			Add(trie, "p228", 					{1,9,13,52,1,600,0});
			Add(trie, "p90", 					{29,10,50,100,0,2350,0});
			Add(trie, "scout", 					{3,2,10,90,0,2750,0});
			Add(trie, "sg550", 					{13,3,30,90,0,4200,0});
			Add(trie, "sg552", 					{26,3,30,90,0,3500,0});
			Add(trie, "tmp", 					{22,6,30,120,0,1250,0});
			Add(trie, "ump45", 					{12,8,25,100,0,1700,0});
			Add(trie, "usp", 					{16,8,12,100,1,500,0});
			Add(trie, "xm1014", 				{5,7,7,32,0,3000,0});
			
			Add(trie, "knife", 					{28,-1,-1,0,2,0,0});
			
			Add(trie, "hegrenade", 				{4,11,1,0,3,300,0});
			Add(trie, "hegren", 				{4,11,1,0,3,300,0});
			
			Add(trie, "flashbang", 				{24,12,1,0,3,200,0});
			Add(trie, "flash", 					{24,12,1,0,3,200,0});
			
			Add(trie, "smokegrenade", 			{9,13,1,0,3,300,0});
			
			Add(trie, "c4", 					{6,-1,-1,0,4,0,0});
		}
		
		default: {
			SetFailState("This game is not supported");
		}
	}
}